#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define _MAINVARIABLES_
#include <Main.h>
#include <Display2.h>
#include <Utilities2.h>
#include <Howard.h>
#include <Karp4.h>
#include <time.h>
#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC 1000000
#endif

void Allocate_Main_Variables(int MAXSTAT,int NARCSPERNODE)
{
   ij=  (int *)calloc(MAXSTAT*NARCSPERNODE, 2*sizeof(int));
   pi=  (int *)calloc(MAXSTAT, sizeof(int));
   a= (double *)calloc(MAXSTAT*NARCSPERNODE, sizeof(double));
   v= (double *)calloc(MAXSTAT, sizeof(double));
   chi= (double *)calloc(MAXSTAT, sizeof(double));
   if ((chi==NULL) || (ij==NULL) || (a==NULL)|| (v==NULL))
    {
      printf("Error in Main... memory allocation failed...\n");
      exit(1);
    }
}

void Semi_Allocate_Main_Variables(int MAXSTAT,int NARCSPERNODE)
{
   Allocate_Main_Variables(MAXSTAT,NARCSPERNODE);
   t= (double *)calloc(MAXSTAT*NARCSPERNODE, sizeof(double));
   if (t==NULL)
    {
      printf("Error in Main... memory allocation failed...\n");
      exit(1);
    }

}
void  Compare_Karp_With_Howard(double u,double c,int narcs,int *ij,double *a)
{
	   if ((u-chi[0])>0.1 || (u-chi[0])<-0.00001)
	     {
	          printf("ERROR: KARP AND HOWARD DIFFER\n");
	          Display_Sparse_Matrix(narcs,ij,a);
	          printf("value =\n");
		  Display_Vector(nnodes,v);  
		  printf("chi =\n");
		  Display_Vector(nnodes,chi); 
  		  printf("Value of Karp (=%f)\n",u);
		  exit(1);
	     }
}


static void Update(double *NItMEAN,double *NItMIN,double *NItMAX, int *MAXABS,int NIterations)
{
	  *NItMEAN=*NItMEAN+NIterations;
          if (NIterations>*NItMAX)
	    {
	      *NItMAX=NIterations;
	    }
	  if (NIterations<*NItMIN)
	    {
	      *NItMIN=NIterations;
	    }
          if (NIterations>*MAXABS)
	    {
	      *MAXABS=NIterations;
	    }
}

void main()
{  int N;
   int iaux;
   int verbosemode;
   double u;
   double NItMEAN;
   double NItMIN;
   double NItMAX;
   double TIME;
   long LASTTIME=0,NEWTIME=0;
   int STEP;
   int FS,KARPORHOWARD;
   int MINSTAT,MAXSTAT,NSAMPLE,NARCSPERNODE,NIterations,MAXABS=0;
   printf("# minimal dimension =? ");
   scanf("%d",&MINSTAT);
   printf("# maximal dimension =? ");
   scanf("%d",&MAXSTAT);
   printf("# step =? ");
   scanf("%d",&STEP);
   printf("# full (0) or sparse (1) ? ");
   scanf("%d",&FS);
   if (FS !=0) 
     {
       printf("# number of arcs per node =? ");
       scanf("%d",&NARCSPERNODE);
     }
   else 
     NARCSPERNODE=MAXSTAT;
   
   printf("# Howard (0) or Karp (1) or both (2) or generate lp_solve input (3)..\n");
   printf("# or SemiHoward (4) or Big Karp (5) or Howard and Big Karp (6) ? ");
   scanf("%d",&KARPORHOWARD);
   printf("# number of matrices tested per dimension =? ");
   scanf("%d",&NSAMPLE);
   printf("# verbosemode =  (normal 0, the higher, the more verbose) ? ");
   scanf("%d",&verbosemode);
   if (KARPORHOWARD !=4) 
     {
       Allocate_Main_Variables(MAXSTAT,NARCSPERNODE);
     }
   else
     {
       Semi_Allocate_Main_Variables(MAXSTAT,NARCSPERNODE);
     }
   printf("#         ********         STATISTICS      ********\n");
   printf("# NN       = number of nodes\n");
   printf("# NA       = number of arcs\n");
   printf("# mean_I   = mean number of iterations in the current sample\n");
   printf("# min_I    = min number of iterations ...\n");
   printf("# max_I    = max number of iterations ...\n");
   printf("# T_max_I  = max number of iterations from the begining\n");
   printf("# TIME     = mean cpu time (in sec) per call of Howard\n");
   printf("#         *****************************************\n");
   printf("# NN\tNA\tmean_I\tmin_I\tmax_I\tT_max_I\tTIME\n");
   for (nnodes=MINSTAT;nnodes<MAXSTAT+1;nnodes=nnodes+STEP)
     {
       switch (FS){
       case 0:
	 N=nnodes;
	 break;
       default:
	 N=NARCSPERNODE;
	 break;
       }
       narcs=N*nnodes;
       NItMEAN=0;
       NItMIN=HUGE_VAL;
       NItMAX=-HUGE_VAL;
       for (iaux=0;iaux<NSAMPLE; iaux++)
	 {
       if (KARPORHOWARD!=4)
	 {
	   Make_Rand_Sparse_Matrix(nnodes,N,ij,a);
	   if (verbosemode >0)
	    {
	      Display_Sparse_Matrix(narcs,ij,a);
	    }
	 }
       else
	 {
	   Semi_Make_Rand_Sparse_Matrix(nnodes,N,ij,a,t);
   	   if (verbosemode >0)
	    { printf("CURRENT MATRIX:\n");
	      Display_Semi_Sparse_Matrix(narcs,ij,a,t);
	    }

	 }
       switch (KARPORHOWARD){
       case 0: 
        Howard(ij,a,nnodes,narcs,chi,v,pi,&NIterations,&NComponents,verbosemode);
	Update(&NItMEAN,&NItMIN,&NItMAX,&MAXABS,NIterations);
	break;
       case 2:	 
        Howard(ij,a,nnodes,narcs,chi,v,pi,&NIterations,&NComponents,verbosemode);
        Karp(ij,a,nnodes,narcs,0,&u);
        Compare_Karp_With_Howard(u,chi[0],narcs,ij,a);
	Update(&NItMEAN,&NItMIN,&NItMAX,&MAXABS,NIterations);
	break;
       case 3:	 
        Howard(ij,a,nnodes,narcs,chi,v,pi,&NIterations,&NComponents,verbosemode);
	Update(&NItMEAN,&NItMIN,&NItMAX,&MAXABS,NIterations);
        Generate_lp_file(narcs,ij,a);
	break;
       case 4:	 
	 {
	   /*	   double remember;
           Howard(ij,a,nnodes,narcs,chi,v,pi,&NIterations,&NComponents,verbosemode);
 	   remember=chi[0];*/
        Semi_Howard(ij,a,t,nnodes,narcs,chi,v,pi,&NIterations,&NComponents,verbosemode);
	/*	printf("remember/2=%g chi[0]=%g\n",remember/2,chi[0]);
	if (abs(remember/2  - chi[0])> 0.00000001)
	  {
	    printf("error : howard and semihoward differ\n");
	    exit(1);
	  }
	  */
	Update(&NItMEAN,&NItMIN,&NItMAX,&MAXABS,NIterations);
	 }
	break;
       case 5:
        Karp_Big(ij,a,nnodes,narcs,0,&u);
	Update(&NItMEAN,&NItMIN,&NItMAX,&MAXABS,NIterations);
	break;
       case 6:
	Howard(ij,a,nnodes,narcs,chi,v,pi,&NIterations,&NComponents,verbosemode);
        Karp_Big(ij,a,nnodes,narcs,0,&u);
        Compare_Karp_With_Howard(u,chi[0],narcs,ij,a);
	Update(&NItMEAN,&NItMIN,&NItMAX,&MAXABS,NIterations);
	break;
       default:
         Karp(ij,a,nnodes,narcs,0,&u);
/*	 printf("RESULT OF KARP :  u=%f\n",u);*/
       }
	 }
       NEWTIME=clock();
       TIME=((double)(NEWTIME -LASTTIME))/CLOCKS_PER_SEC/NSAMPLE;
/*       printf("%d %f %d %g\n",NEWTIME,(double)(NEWTIME-LASTTIME),CLOCKS_PER_SEC,TIME);*/
       LASTTIME=NEWTIME; 
       /* produces an output suitable for plotting */
       printf("[%d,\t%d,\t%g,\t%g,\t%g,\t%d,\t%f],\\ \n",nnodes,narcs,NItMEAN/NSAMPLE,NItMIN,NItMAX,MAXABS,TIME);
     }
}
/****************************************************************/











